

/***********************************************************************
 *                                                                     *
 *             S e r i a l  c o m m u n i c a t i o n                  *
 *                     c o m p o n e n t                               *
 *                                                                     *
 *                Copyright (C) Andreas Heiming 2002                   *
 *                   All rights reserved                               *
 *                                                                     *
 ***********************************************************************

 Description

 Author:               Andreas Heiming

 Version:              1.1

 Creation Date:        13-Okt-2002



 List of modifications:

 name:    date:       modification:

 */

#ifdef __BORLANDC__
#include <vcl.h>
#pragma hdrstop
#endif

#include "commhdl.h"

/* ------------------------------------------------------------------------------ */

const char* aCtrlChr[32] = {"NUL","SOH","STX","ETX","EOT","ENQ","ACK","BEL","BS",
            "HT","LF","VT","FF","CR","SO","SI","DLE","DC1","DC2",
            "DC3","DC4","NAK","SYN","ETB","CAN","EM","SUB","ESC",
            "FS","GS","RS","US"};

void  TCommPort::SetReceiveTimeoutLength(int& AValue)
{
  COMMTIMEOUTS CommTimeouts;
  iReceiveTimeoutLength=AValue;
  if (iCommId!=NULL)
    {
      GetCommTimeouts(iCommId,&CommTimeouts);  
      CommTimeouts.WriteTotalTimeoutMultiplier=1;
      CommTimeouts.WriteTotalTimeoutConstant=iSendTimeoutLength;
      CommTimeouts.ReadIntervalTimeout=10;
      CommTimeouts.ReadTotalTimeoutMultiplier=1;
      CommTimeouts.ReadTotalTimeoutConstant=iReceiveTimeoutLength;
      SetCommTimeouts(iCommId,&CommTimeouts);      
    }
}

void  TCommPort::SetSendTimeoutLength(int& AValue)
{
  COMMTIMEOUTS CommTimeouts;

  iSendTimeoutLength=AValue;
  if (iCommId != NULL)
  {
      GetCommTimeouts(iCommId,&CommTimeouts);  
      CommTimeouts.WriteTotalTimeoutMultiplier=1;
      CommTimeouts.WriteTotalTimeoutConstant=iSendTimeoutLength;
      CommTimeouts.ReadIntervalTimeout=10;
      CommTimeouts.ReadTotalTimeoutMultiplier=1;
      CommTimeouts.ReadTotalTimeoutConstant=iReceiveTimeoutLength;
      SetCommTimeouts(iCommId,&CommTimeouts);      
  }
}

TCommPort::TCommPort()
{
  ComPort=1;
  cStartChar=0;
  cEndChar=LF;
  cDupChar = 0;
  cXOffChar=XOFF;
  cXOnChar=XON;
  iXonLimit=10;
  iXoffLimit=10;
  fHWFlowOptions = last_THWFlowOptions;
  fSWFlowOption=cswfNone;
  bLogFileOpen=false;
  sLogFilename="COMMHDL.LOG";
  iStartTime=GetTickCount();
  iLastAccessTime=iStartTime;
  fLogFileMode=lfmOverwrite;
  bWithStartChar=false;
  bWithEndChar=false;
  bWithAnswer = false;
  fProtocol=cpcNone;
  iReceiveTimeoutLength=1000;
  iSendTimeoutLength=1000;
  iBaudrate=19200;
  iDatabits=8;
  iInSize=1024;
  iOutSize=1024;
  iRetries=3;
}



TCommPort::~TCommPort()
{
  Close();
  CloseLogFile();
}



int  TCommPort::GetCommError()
{
  DWORD dError;
  COMSTAT rComStat;
  ClearCommError(iCommId,&dError,&rComStat);
  return dError;
}

int  TCommPort::GetLastError()
{
  return iLastError;
}

HANDLE  TCommPort::GetHandle()
{
  return iCommId;
}

void  TCommPort::ClearSendBuffer()
{
  if (iCommId>0)
    PurgeComm(iCommId,PURGE_TXCLEAR);
//  else

//    raise EError.Create('CommHandler not open ! (ClearSendBuffer)');

}



void  TCommPort::ClearReceiveBuffer()
{
  if (iCommId>0) 
    PurgeComm(iCommId,PURGE_RXCLEAR);
//  else

//    raise EError.Create('CommHandler not open ! (ClearReceiveBuffer)');

}

void  TCommPort::SetParameter(int& pComPort,
                                          int& pBaudRate,
                                          int& pDataBits,
                                          TParity& pParity,
                                          TStopbits& pStopbits)
{
  ComPort=pComPort;
  iBaudrate=pBaudRate;
  iDatabits=pDataBits;
  fParity=pParity;
  fStopbits=pStopbits;
}

bool  TCommPort::Open()
{
  COMMTIMEOUTS CommTimeouts;
  char sComNo[20];
  LPVOID lpMsgBuf;

  sprintf(&sComNo[0],"Com%d",ComPort);
  iCommId = CreateFile(sComNo,              /* lpFileName */
                         GENERIC_READ | GENERIC_WRITE,  /* dwDesiredAccess */
                         0,                             /* dwShareMode */
                         0,                             /* lpSecurityAtributes */
                         OPEN_EXISTING,                 /* dwCreationDistribution */
                         0,                             /* dwFlagsAndAttributes */                   /* dwFlagsAndAttributes */
                         0);                            /* hTemplateFile */

  if (iCommId < 0)
  {
    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                    NULL,
                    GetLastError(),
                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                    (LPTSTR) &lpMsgBuf,
                    0,
                    NULL);

    // Display the string.
    MessageBox( NULL, (LPTSTR) lpMsgBuf, "GetLastError", MB_OK|MB_ICONINFORMATION );
    // Free the buffer.
    LocalFree( lpMsgBuf );
    return false;
  }
  else
    {
      bool t = GetCommState(iCommId, &fDCB);
      memset(&fDCB,OFF,sizeof(fDCB));

      /* Setze die Flags im DCB */
      fDCB.fBinary	= ON;

      if (fHWFlowOptions == chwfUseRTS)
         fDCB.fRtsControl == ON;
      if (fHWFlowOptions == chwfUseDTR)
         fDCB.fDtrControl == ON;
      if (fHWFlowOptions == chwfRequireCTS)
      {
          fDCB.fOutX			= ON;
    	  fDCB.fInX			= ON;
      }
       fDCB.fInX=true;  // fr Empfnger
       fDCB.fOutX=true; // fr Sender


      if (fHWFlowOptions == chwfRequireDSR)
        fDCB.fOutxDsrFlow |= 1;

      fDCB.XoffChar=cXOffChar;
      fDCB.XonChar=cXOnChar;

      fDCB.XonLim=iXonLimit;
      fDCB.XoffLim=iXoffLimit;
      // kein Hardware Flowcontrol:
      fDCB.fOutxCtsFlow=false;
      fDCB.fOutxDsrFlow=false;

      fDCB.fDsrSensitivity=false;

      if ((fSWFlowOption == cswfBoth) || (fSWFlowOption == cswfReceive))
        fDCB.fInX |= 1;
      if ((fSWFlowOption == cswfBoth) || (fSWFlowOption == cswfTransmit))
        fDCB.fOutX |= 1;

      fDCB.BaudRate=iBaudrate;
      fDCB.ByteSize=iDatabits;

      switch (fStopbits) {
        case csb1   : fDCB.StopBits=ONESTOPBIT; break;
        case csb1_5 : fDCB.StopBits=ONE5STOPBITS; break;
        case csb2   : fDCB.StopBits=TWOSTOPBITS; break;
      }

      switch(fParity)
      {
        case cpNone:
          fDCB.Parity = NOPARITY;
          break;
        case cpOdd:
          fDCB.Parity = ODDPARITY;
          break;
        case cpEven:
          fDCB.Parity = EVENPARITY;
          break;
        case cpMark:
          fDCB.Parity = MARKPARITY;
          break;
      }
      if (fParity>cpNone)
        fDCB.fParity |= 1;

      fDCB.DCBlength	= sizeof(fDCB);
      /* Setze DCB */
      try {
        if(SetCommState(iCommId,&fDCB) == false)
        {
         iLastError = GetCommError();
         FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                    NULL,
                    iLastError,
                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                    (LPTSTR) &lpMsgBuf,
                    0,
                    NULL);
           // Display the string.
          MessageBox(NULL , (LPTSTR) lpMsgBuf, "GetLastError", MB_OK|MB_ICONINFORMATION );

          // Free the buffer.
          LocalFree( lpMsgBuf );
          return false;
        }
      }
      catch(exception &e)
      {
         throw(e.what());
//        throw("COMM - Error : SetCommState");
      }
      
      if(SetupComm(iCommId,iInSize,iOutSize) == false)
      {
        iLastError = GetCommError();
        FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                    NULL,
                    iLastError,
                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                    (LPTSTR) &lpMsgBuf,
                    0,
                    NULL);
         // Display the string.
        MessageBox(NULL , (LPTSTR) lpMsgBuf, "GetLastError", MB_OK|MB_ICONINFORMATION );

        // Free the buffer.
        LocalFree( lpMsgBuf );
        return false;
      }
      
      GetCommTimeouts(iCommId,&CommTimeouts);  
      CommTimeouts.WriteTotalTimeoutMultiplier=1;
      CommTimeouts.WriteTotalTimeoutConstant=iSendTimeoutLength;
      CommTimeouts.ReadIntervalTimeout=10;
      CommTimeouts.ReadTotalTimeoutMultiplier=1;
      CommTimeouts.ReadTotalTimeoutConstant=iReceiveTimeoutLength;
      SetCommTimeouts(iCommId,&CommTimeouts);      
      return true;
  }
}



bool  TCommPort::Close()
{
  if(CloseHandle(iCommId))
    return true;
  else
    return false;  
}



char  TCommPort::GetChar()
{
  char cCharacter;
  DWORD iReadCount;
  bool bResult;


  bResult=ReadFile(iCommId,&cCharacter,1,&iReadCount,NULL);
  if (bResult)
    if (iReadCount==0)
      iLastError = CE_TIMEOUT;
    else
      iLastError = CE_OK;
  else
    iLastError = GetCommError();

  if (iReadCount>0)  return cCharacter; else return '\0';
}

void  TCommPort::SendChar(char Ch)
{
  DWORD dWrCount;

  WriteFile(iCommId,&Ch,1,&dWrCount,NULL);
  WriteLog(Ch,ltWrite);
}


void  TCommPort::CloseLogFile()
{
  char Datumsstring[20];
  char Zeitstring[40];
  struct tm *AktuellesDatum;

  if (bLogFileOpen)
  {
     time_t ZeitDatum;
     ZeitDatum = time(NULL);				// Systemaufruf
     AktuellesDatum = localtime(&ZeitDatum);
     sprintf(Datumsstring,"%i.%i.%i",AktuellesDatum->tm_mday, 
                                     AktuellesDatum->tm_mon, 
                                     AktuellesDatum->tm_year);     
     sprintf(Zeitstring,"%d:%d:%d",AktuellesDatum->tm_hour,AktuellesDatum->tm_min,AktuellesDatum->tm_sec);
     fprintf(tfLogFileHandle,"%s","-------------------------------------------------------\n");
     fprintf(tfLogFileHandle,"%s%s","LOG-File closed",". Date : ");
     fprintf(tfLogFileHandle,"%s%s%s",Datumsstring," Time : ",Zeitstring);
     fprintf(tfLogFileHandle,"%s","-------------------------------------------------------\r\n");
     fclose(tfLogFileHandle);
     bLogIt=false;
     bLogFileOpen=false;
  }
}

void  TCommPort::OpenLogFile()
{
  char* ModeText;
  char Datumsstring[20];
//	char* Zeitstring;
	struct tm *AktuellesDatum;

	time_t ZeitDatum;
	ZeitDatum = time(NULL);				// Systemaufruf
	AktuellesDatum = localtime(&ZeitDatum);
    sprintf(Datumsstring,"%i.%i.%i",AktuellesDatum->tm_mday, 
                                     AktuellesDatum->tm_mon, 
                                     AktuellesDatum->tm_year);     

  try
  {
    if (fLogFileMode==lfmAppend)
    {
		tfLogFileHandle = fopen(sLogFilename.c_str(), "a");
      ModeText="appended";
    }
    else
    {
  		tfLogFileHandle = fopen(sLogFilename.c_str(), "w");
      ModeText="created";
    }

  	char* Zeitstring = new char[80];
	  bLogFileOpen=true;
    sprintf(Zeitstring,"%2d:%2d:%2i",AktuellesDatum->tm_hour,AktuellesDatum->tm_min,AktuellesDatum->tm_sec);
	  fprintf(tfLogFileHandle,"%s","-------------------------------------------------------\n");
  	fprintf(tfLogFileHandle,"%s%s%s","LOG-File ",ModeText,". Date : ");
	 	fprintf(tfLogFileHandle,"%s%s%s%s",Datumsstring," Time : ",Zeitstring," \n");
  	fprintf(tfLogFileHandle,"%s","-------------------------------------------------------\n");
  	bLogIt = true;
//		delete Zeitstring;
  }
  catch(...)
  {
    bLogIt=false;
    throw("Cant open Logfile !");
  }
}


void TCommPort::WriteLog(char Text, int LogType)
{
  string tmpText;
  tmpText[0] = Text;
  WriteLog(tmpText, LogType);
}

string  CtrlChrToText(string& Text)
{
  int i;

  string CtrlChrToText_result = "";

  for( i=0; i < (int) Text.length(); i ++)
    if (Text[i]>'\37') 
      CtrlChrToText_result=CtrlChrToText_result+Text[i];
    else
      CtrlChrToText_result=CtrlChrToText_result+'['+aCtrlChr[(int)(Text[i])]+']';
  return CtrlChrToText_result;
}

void TCommPort::WriteLog(string Text, int LogType)
{
  string sLogText;
  DWORD iDelta;
  char Datumsstring[20];
//	char* Zeitstring;
	struct tm *AktuellesDatum;
	string TextString;

	time_t ZeitDatum;
	ZeitDatum = time(NULL);				// Systemaufruf
	AktuellesDatum = localtime(&ZeitDatum);
    sprintf(Datumsstring,"%i.%i.%i",AktuellesDatum->tm_mday, 
                                    AktuellesDatum->tm_mon, 
                                    AktuellesDatum->tm_year);     

  if (bLogIt)
  {
		if (! bLogFileOpen)  OpenLogFile();

    if (bLogIt && ((fLogFileType==lftAll) || ((fLogFileType==lftOnlyErrors) &&
        (iLastError==CE_OK))))
    {
    	if (LogType==ltWrite)
      	sLogText="Write";
      else
        sLogText="Read ";

//      iDelta=GetTickCount()-iLastAccessTime;
        iLastAccessTime=GetTickCount();

  	    ZeitDatum = time(NULL);				// Systemaufruf
	    AktuellesDatum = localtime(&ZeitDatum);

  		char* Zeitstring = new char[80];
    	sprintf(Zeitstring,"%2d:%2d:%2i",AktuellesDatum->tm_hour,AktuellesDatum->tm_min,AktuellesDatum->tm_sec);
  		TextString = CtrlChrToText(Text);
	  	fprintf(tfLogFileHandle,"%s%s%s",Datumsstring," Time : ",Zeitstring);
      fprintf(tfLogFileHandle," %s%s",TextString.c_str(),"\n");
    }
  }
}


