#ifndef __commhdl_h__
#define __commhdl_h__

#include <string>
#include <stdio.h>
#include <exception> 
#include <time.h> 
#include <Windows.h>
#include <Winbase.h>
using namespace std;
#pragma hdrstop

#define		OFF					0x00
#define		ON					0x01

enum TStopbits {  csb1, csb1_5, csb2, last_TStopbits};
enum TParity {    cpNone, cpOdd, cpEven, cpMark, cpSpace, last_TParity};
enum TProtocol {  cpcNone, cpcAckNak, cpcTLP1, cpcTLP2, cpcAckNakComplement, cpcTTY, cpcDust3964R, cpcUser, last_TProtocol};
enum TCheckSum {  ccsNone, ccsComplement, ccsXOR, ccsTLP2, last_TCheckSum};
enum TLogFileType {lftAll, lftOnlyErrors, last_TLogFileType};
enum TLogFileMode {lfmAppend, lfmOverwrite , last_TLogFileMode};
enum THWFlowOptionSet {chwfUseDTR, chwfUseRTS, chwfRequireDSR, chwfRequireCTS, last_THWFlowOptions};
enum TSWFlowOption {cswfNone, cswfBoth, cswfReceive, cswfTransmit, last_TSWFlowOption};

//     THWFlowOptionSet = set of (THWFlowOptions);

// class ECommException : public Exception{};

class TCommPort
{
  protected:
    int ComPort;
    int iBaudrate;
    TParity fParity;
    TStopbits fStopbits;
    int iDatabits;

    int iXonLimit,
    iXoffLimit,
    iInSize,
    iOutSize,
    iRetries;

    DWORD iWrCount;
    TLogFileMode fLogFileMode;
    TCheckSum fCheckSum;
    TProtocol fProtocol;

    bool bWithEndChar,
    bWithStartChar,
    bWithAnswer,
    bLogIt,
    bLogFileOpen;

    TLogFileType fLogFileType;

    FILE *tfLogFileHandle;

    int iLastError;    
    int iReceiveTimeoutLength,
    iSendTimeoutLength;
    HANDLE iCommId;

    char cXOnChar,
    cXOffChar,
    cDupChar;
    char cStartChar,
    cEndChar;

    string sLogFilename;
    DCB fDCB;

    int iStartTime,
    iLastAccessTime;

    THWFlowOptionSet fHWFlowOptions;
    TSWFlowOption fSWFlowOption;

    TProtocol GetProtocol();

    void  WriteLog(string Text, int LogType);
    void  WriteLog(char Text, int LogType);

    int  GetCommError();

  public:
      TCommPort();
      virtual ~TCommPort();

    void  SetReceiveTimeoutLength(int& AValue);
    void  SetSendTimeoutLength(int& AValue);
    void   SetParameter(int& pComPort,
                                 int& pBaudRate,
                                 int& pDataBits,
                                 TParity& pParity,
                                 TStopbits& pStopbits);

    void  OpenLogFile();
    void  CloseLogFile();

    bool  Open();
    bool  Close();

    char  GetChar();
    char  WaitForChar(char WaitChar);

    void  SendChar(char Ch);

    void  ClearSendBuffer();
    void  ClearReceiveBuffer();
    int   GetLastError();
    HANDLE GetHandle();
};

const int dcbfBinary = 1;             // binary mode, no EOF check
const int dcbfParity = 2;          // enable parity checking
const int dcbfOutxCtsFlow = 4;          // CTS output flow control
const int dcbfOutxDsrFlow = 8;          // DSR output flow control
const int dcbfDtrControlEnable = 16;         // DTR flow control type
const int dcbfDtrControlHandshake = 32;
const int dcbfDsrSensitivity = 64;         // DSR sensitivity
const int dcbfTXContinueOnXoff = 128;        // XOFF continues Tx
const int dcbfOutX = 256;        // XON/XOFF out flow control
const int dcbfInX = 512;        // XON/XOFF in flow control
const int dcbfErrorChar = 1024;       // enable error replacement
const int dcbfNull = 2048;       // enable null stripping
const int dcbfRtsControlEnable = 4096;
const int dcbfRtsControlHandshake = 8192;       // RTS flow control
const int dcbfRtsControlToggle = 12288;
const int dcbfAbortOnError = 16384;      // abort reads/write =s on error


const int ltRead = 0;
const int ltWrite = 1;

const char NUL = '\0'; const char SOH = '\1'; const char STX = '\2'; const char ETX = '\3'; const char EOT = '\4'; const char ENQ = '\5';
const char ACK = '\6'; const char BEL = '\7'; const char BS = '\10'; const char HT = '\11'; const char LF = '\12'; const char VT = '\13';
const char FF = '\14'; const char CR = '\15'; const char SO = '\16'; const char SI = '\17'; const char DLE = '\20'; const char DC1 = '\21';
const char DC2 = '\22'; const char DC3 = '\23'; const char DC4 = '\24'; const char NAK = '\25'; const char SYN = '\26'; const char ETB = '\27';
const char CAN = '\30'; const char EM = '\31'; const char SUB = '\32'; const char ESC = '\33'; const char FS = '\34'; const char GS = '\35';
const char RS = '\36'; const char US = '\37';

const char XOFF = '\23'; const char XON = '\21';

const int CE_OK = 0;
const int CE_TIMEOUT = CE_PTO;
const int CE_NAKRECEIVED = -1;
const int CE_NAKTRANSMIT = -2;

void GetCommList(int& ComPortsToCheck, string& CommList);

#endif
